@@ -198,6 +198,15 @@ bool Game::simpleCheck() const { // IDW TODO - This does nothing useful now
 // IDW test. Eliminated optimized[] arrays and xxxConnection() functions.
 }
 
+void Game::restart() {
+	while (canUndo()) {
+		interface()->undo();
+	}
+	m_private->history.clear(); // otherwise we could do redo
+	m_private->wasFinished = false;
+	m_private->emitModified(true); // e.g. to update undo/redo action state
+}
+
 int Game::order() const {
 	if(!m_private) return 0;
 	return m_private->puzzle->order();
@@ -566,14 +575,23 @@ void Game::deleteCageAt (int pos, SKGraph * g)
 	}
 }
 
+bool Game::allValuesSetAndUsable() const {
+	for (int i = 0; i < size(); i++) {
+		if (value(i) == 0) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 void Game::checkCompleted() {
 	if(!m_private || !m_private->puzzle->hasSolution()) return;
 
-	// Find cells that are empty and not in unusable areas (as in Samurai).
-	for(int i = 0; i < size(); i++)
-		if((value(i) == 0) && (solution(i) > 0))
-			return;
-	
+	if (!allValuesSetAndUsable()) {
+		return;
+	}
+
 	for(int i = 0; i < size(); i++) {
 		if(value(i) != solution(i)) {
 			m_private->emitCompleted(false, time(), m_private->hadHelp);
